# get parameter from command line
Param( [Parameter(Mandatory=$false)] [switch]$d )
#
#
# PowerShell Scrip to analyze a SEP sesam yesterday's status file and
# report backup status to Check-MK.
#
# (c) 2020 by Walter@Hofstaedtler.com, all rights reserved.
#
#  Based on an idea of Matthias Henze https://wiki.mhcsoftware.de/sep_sesam_check.
#
# Modified and ported to Windows by Walter Hofstaedtler.
# This is free software, but use it on your own risk.
#
# V.: 0.1, 02.05.2020 WH, Initial Windows release by WH
# V.: 0.2, 05.05.2020 WH, small improvements
# V.: 0.3, 07.05.2020 WH, added IGNORE_WARNINGS - my be dangerous
#
#
# Tested with:	SEP sesam 4.4.3.72,
#				Check-MK Agent 1.6.x CRE,
#				Windows Server 2019
#
#
# This is a "local check", a "simple script" and outputs 4 fields, separated by space:
# 1:Status 0..3, 2:Service name "SEP-Sesam", 3:Metrics only "-", 4:Status details
# Please see: https://checkmk.de/cms_localchecks.html for explanation.
#
#
# Installation of this script:
#	1. Install Check-MK agent 1.6.x for Windows
#   2. Place this script in folder: %ProgramData%\checkmk\agent\local
#   3. WATO - do a "Full scan" for this host
#   4. WATO - for Undecided services (currently not monitored): "SEP - Sesam" click on "Move to monitored servicese"
#   5. WATO - save new check configuration
#
#
# User settings:
#
# Summarize all successful backups?
#   True:  summarize all OK backups into one item like: OK:15_Jobs
#   False: list all OK jobs separate like: OK:icc-srv, OK:icc-term, OK:test-srv, ...
# True is useful to highlight jobs with error or warning state.
$SUMM_OK = $TRUE
#$SUMM_OK = $FALSE
#
# Ignore warnings?
#   True: report warnings as OK! This can be dangerous!
#   False: report warnings as warnings.
# Ignoring warnings can be dangerous, its always wise to fix warnings to have only successful backups
$IGNORE_WARNINGS = $FALSE
#$IGNORE_WARNINGS = $TRUE
#
# ----- Debug settings only, do not change - start ----
#
# This script accepts only one parameter: "-d" for debug output.
#
# Debug Log:
# Set empty for no logging, please use logging only for debugging.
#$LOG = "C:\TEMP\check-mk_sesam.log"
$LOG = ""
# ----- Debug settings only, do not change - end ----

# --- Program starts here ---
#
$WH_DEBUG = $FALSE
# Debug switch in command line? If yes, set $WH_DEBUG = $TRUE
if ($d) { $WH_DEBUG = $TRUE }
if ($WH_DEBUG) { Write-Output " D: Command line parameter `$d: $d ." }

# Debug log start
if ($LOG -ne "") { Write-Output "$(Get-Date -Format "dd.MM.yyyy HH:MM:ss") Start script: $MyInvocation.MyCommand.Name " >> $LOG }

# Select x64 or i386 version
if (Test-Path "HKLM:\SOFTWARE\SEP Elektronik GmbH")
{
   $ini = ((get-itemproperty 'HKLM:\SOFTWARE\SEP Elektronik GmbH\sesam\GLBV\gv_rw_ini')."(default)").Replace("/","\")
} else {
   $ini = ((get-itemproperty 'HKLM:\SOFTWARE\WOW6432Node\SEP Elektronik GmbH\sesam\GLBV\gv_rw_ini')."(default)").Replace("/","\")
}

# Execute SEP sesam profile
. $ini\sm_prof.ps1

# Get language string
$lang = (Get-Content -Path $ini\sm.ini | Where-Object { $_ -match 'lang=' }).Split('=')[1]

# Debug:
if ($WH_DEBUG) { Write-Output " D: Ini: $ini, Lang: $lang, gv_rw_prot: $env:gv_rw_prot, env:prot: $env:prot ." }

# Get yesterday's status file
$File = (Get-Childitem -Path $env:prot\*.status | Sort-Object -Property CreationTime -Descending )[1]

# Init variables used in ForEach-Object ...
$STATUS = 0
$TEXT = ""
$OK_CNT = 0
$ADD_COMMA = $FALSE

# Debug
if ($WH_DEBUG) { Write-Output " D: Analyze yesterday's status file: $FILE" }
if ($LOG -ne "") { Write-Output "$(Get-Date -Format "dd.MM.yyyy HH:MM:ss") Yesterday's status file: $FILE " >> $LOG }

# ignore "^Start|^-----" but $regex do not work, removed in switch ($S)
# $regex = "^Start|^-----"
Get-Content $FILE | Where-Object {$_ -match $regex} | ForEach-Object {
	# Get S - status code
	$S = $_.SubString(0,1)
	# Get N - backup job name, max 51 characters long
	$POS = $($_.SubString(2,53)).IndexOf(' ')
	if ( $POS -lt 1 ) { $POS = 2 }
	$N = $_.SubString(2, $POS)
	if ($WH_DEBUG) { Write-Output " D: Pos: $POS, S: $S, N: $N, `$_: $($_.SubString(0,70))" }

	# add a comma after a text segment
	if ( $ADD_COMMA ) {$TEXT = "$TEXT, "}
	$ADD_COMMA = $TRUE

	switch ( $S )
	{
		0 	{ # successful
				$OK_CNT++
				if ($SUMM_OK) { $ADD_COMMA = $FALSE	} else { $TEXT = $TEXT + "OK:" + $N }
			}
		1 	{ # successful with warning
				if ( $STATUS -lt 1 ) { $STATUS = 1 }
				$TEXT = $TEXT + "WARN:" + $N
			}
		2	{ # listing incomplete
				if ( $STATUS -lt 2 ) { $STATUS = 2 }
				$TEXT = $TEXT + "INCOMP:" + $N
			}
		3	{ # aborted
				if ( $STATUS -lt 2 ) { $STATUS = 2 }
				$TEXT = $TEXT + "ABORT:" + $N
			}
		c	{ # aborted by user or server
				if ( $STATUS -lt 2 ) { $STATUS = 2 }
				$TEXT = $TEXT + "USER:" + $N
			}
		X	{ # fatal error
				if ( $STATUS -lt 2 ) { $STATUS = 2 }
				$TEXT = $TEXT + "FATAL:" + $N
			}
		S	{ # first line - ignore
				$ADD_COMMA = $FALSE
			}
		-	{ # second line - ignore
				$ADD_COMMA = $FALSE
			}
		default { # unknown
				$STATUS = 3
				$TEXT = $TEXT + "UNKNOWN:" + $N
			}
	} # switch ( $S )
} # Get-Content $FILE

if ( $SUMM_OK ) {
	if ( $OK_CNT -eq 1 ) { $MZ = "" } else { $MZ = "s" }
	if ($WH_DEBUG) { Write-Output " D: Sum of successful jobs: OK: $OK_CNT Job$MZ." }
	$TEXT = $TEXT + "OK:" + $OK_CNT + "_Job" + $MZ + "."
}

# Ignoring warnings can be dangerous
if ( $IGNORE_WARNINGS ) {
	if ($WH_DEBUG) { Write-Output " D: `$IGNORE_WARNINGS is activated, `$STATUS: $STATUS." }
    if ( $STATUS -eq 1 ){
		# Reset status to 0 == successful 
		$STATUS = 0
   	    if ($WH_DEBUG) { Write-Output " D: `$IGNORE_WARNINGS is activated, reset `$STATUS to 0." }
    }
}

# Split status line into array
$DayArray = $(Get-Content $FILE -First 1).Split(" ")

if ($WH_DEBUG) {
	# for debugging, show all elements of status line array
	$cnt = 0
	foreach ($detailexplain in $DayArray){
		Write-Output " D: Element $cnt is $detailexplain"
		$cnt++
	}
} # if ($WH_DEBUG)

# generate output for Check-MK
switch ( $lang )
{
	"de" { # de string
		if ($WH_DEBUG) { Write-Output " D: Language DE detected." }
		$OUT = "$STATUS SEP-Sesam - Day: $($DayArray[6]) $($DayArray[7]) ($($DayArray[9]), $TEXT"
	}
	"en" { # en string - not tested
		if ($WH_DEBUG) { Write-Output " D: Language EN detected - not tested." }
		$OUT = "$STATUS SEP-Sesam - Day: $($DayArray[8]) $($DayArray[9]) $($DayArray[10]) $($DayArray[11]) ($($DayArray[13]), $TEXT"
	}
	default { # ?? string - not tested
		if ($WH_DEBUG) { Write-Output " D: Unknown language $lang detected." }
		$OUT = "$STATUS SEP-Sesam - Day: $($DayArray[8]) $($DayArray[9]) $($DayArray[10]) $($DayArray[11]) ($($DayArray[13]), $TEXT"
	}
} # switch ( $lang )

# write status to Check-MK
Write-Output "$OUT"
# write output to debug log
if ($LOG -ne "") { Write-Output "$(Get-Date -Format "dd.MM.yyyy HH:MM:ss") Output: $SOUT" >> $LOG }

# Debug logging, terminate log file
if ($LOG -ne "") { Write-Output "$(Get-Date -Format "dd.MM.yyyy HH:MM:ss") End script." >> $LOG }
if ($LOG -ne "") { Write-Output " " >> $LOG }

# --- eof ---